/************************************************************************
 * "Modified by Turbon, Version 0.01.00, 4-9-12 for WM8741 DAC Project" *
 ************************************************************************/



/*
Hifiduino v 0.7a
May 19, 2011: Minor update and bug fixes to the version 0.7 of June 21, 2009

---------------------------------------------------------------------------
----------------------------------------------------------------------------

In this release:
- Support for Web4robot LCD: http://www.arduino.cc/playground/Code/LCDAPI
- Large number display for filter selection and volume
- Digital Filter Selection: Filter 1 to Filter 5.
- Rotary encoder.
- Volume Control. Now -99 db to 0db in 1db increment.
- Volume "Mute". Volume Mutess to -60 dB when pushing rotary encoder
- Changed the default delay in the LCD library to (0,0) See below note.
- Select input sample rate (low, med, high) for future use.
- Adjust brightness and contrast of display with remote. Not used
- Remember brightness and contrast settings in eeprom. Not used
- New fonts for large number display
*/

/************************************************************************
 * "Modified by TheGimp, Version 0.01.01, 4-19-12 for WM8741 DAC Project"*
 *  Changed Display to 16 X 4 LCD OPTRIX                                 *
 *  4-20-2012 - Changed Dim to Mute  changed to Beta release B0.01.03    *
 *  4-20-2012 - commented out Apple Remote code, Add new Remote Code     *
 *  Multi Remote code taken from Ken Shirriff's blog                     *
 *  Add array to handle four bytes of data for RC6, use byte 0 for code  *
 *  B0.01.06 Remote works, but keys 6,7,8 operate with up, dn, mute      *
 *  comment out the "if def" statements for all remotes except RC6       *
 *               #ifdef does not work (syntax?)                          *
 ************************************************************************/
/*
- Changed display to 16 X 4
- Removed Apple remote code
- added Sherriff's code for multiple remotes
- added decode for RC6
- added decode for SONY (not sure which one as there are many)
- added placeholder for Apple
- Reorganized code to put all IR Remote operations under one case statement
- Duplicated Button code for filter switch and added to case statement for remote filter operaton
- changed dim to mute and add mute value to voltme value during mute (ie. mute by 30dB)
- added Beta to code revision. now B 00.01.06
*/

// LIBRARIES

#include <LiquidCrystal.h> 			// For LCD
// Initialize the library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 10, 9, 8, 7);
#include <Wire.h> // For I2C (What pins?)
// We will use the eeprom to store values
#include <EEPROM.h>



/*************************************************************************/
/* Codes for remotes to be added here starting with RC6 and Sony         */
/* you will need to add another "elseif to the main for the new remote   */
/* Key code map, Hex first, then Decimal

Key		Sony	Dec		RC6		Dec
Up		2F0		752		41E		1054
Down	AF0		2800	41F		1055
Mute	290		656		40E		1038
1		010		16		401		1025
2		810		2064	402		1026
3		410		1040	403		1027
4		C10		3088	404		1028
5		210		528		405		1029

Sony is weird as the last nibble is 0, doing a shift right by 4
produces the expected results.

RC6 seems to be consistently the lower byte so no shift is necessary for it.

The shift and mask are handled by it statements in the main code.
*/


#define RC6 1

#if defined  RC6
#define  CODEUP 0x1F
#define  CODEDN 0x1E
#define  CODEMU 0x0E
#define  CODE1 0x01
#define  CODE2 0x02
#define  CODE3 0x03
#define  CODE4 0x04
#define  CODE5 0x05
#endif


/*
#define SONY 0x00

#if defined SONY
#define  CODEUP 0x02F
#define  CODEDN 0x0AF
#define  CODEMU 0x029
#define  CODE1 0x001
#define  CODE2 0x081
#define  CODE3 0x041
#define  CODE4 0x0C1
#define  CODE5 0x021

#endif

*/

/* Placeholder for Apple Codes This should be the third byte so shift >> 8 in main code*/
/* These are not the correct codes!!                                                   */

#define APPLE 0x00
/*
#ifdef APPLE
#define  CODEUP 0x02F
#define  CODEDN 0x0AF
#define  CODEMU 0x029
#define  CODE1 0x001
#define  CODE2 0x081
#define  CODE3 0x041
#define  CODE4 0x0C1
#define  CODE5 0x021

#endif

*/


// Define register values, etc to facilitate programming
// Note: Reg address is address plus R/W bit
// This is why the address here is different from the
// address in the data sheet

#define REG9 0x12     				// Register 9 address (Reset)
#define RESET 0x00    				// Write anything to reset the DAC

#define REG7 0x0E     				// Register 7 address (PCM vs DSD)
#define PCMHIGH 0x40  				// Value for reg7 for PCM, High Sample rate for filter
#define PCMMED 0x20  				// Value for reg7 for PCM, Medium Sample rate for filter
#define PCMLOW 0x00  				// Value for reg7 for PCM, Low Sample rate for filter

#define REG6 0x0C     				// Register 6 address (filters)
#define FILTER1 0x00  				// PCM Filter response 1
#define FILTER2 0x01  				// PCM Filter response 2
#define FILTER3 0x02  				// PCM Filter response 3
#define FILTER4 0x03  				// PCM Filter response 4
#define FILTER5 0x04  				// PCM Filter response 5

#define REG0 0x00     				// Register 0 address (Volume Adjustment)
#define REG1 0x02     				// Register 1 address (Volume Adjustment)
#define REG2 0x04     				// Register 2 address (Volume Adjustment)
#define REG3 0x06     				// Register 3 address (Volume Adjustment)
#define REG4 0x08     				// Register 4 address (Volume Control)

// The following to be used in Reg4
//#define VOLCNTR 0x07 				// VolRight=VolLeft, AntiClip, Vol Ramping is on (a)
#define VOLCNTR 0x05  				// VolRight=VolLeft, No AntiClip, Vol Ramping is on (b)

/**********************************************************************************/
//Addd AntiClip as next experiment.
/**********************************************************************************/

//#define MUTE 0x0F 				// Mute and (a) above
#define MUTE 0x0D     				// Mute and (b) above

//#define DEFAULTVOL 0x190 			//-50 dB this is 50x8=400
#define DEFAULTVOL 0x50 			//-10 dB this is 10x8=80
#define MINVOL 0x318    			//-99dB this is 99X8=792. -Dac adjustment is .125 db
#define MAXVOL 0x00     			//-0 dB - Min attenuation
#define MuteVOL 0x0F0    			//-30dB below the default volume level when Muting the volume

#define FILTERPIN 6     			// Button to select Filter
#define VOLMutePIN 5     			// Button for volume Mute feature
#define VOLUPPIN 4      			// Rotary Encoder to increase volume
#define VOLDOWNPIN 2    			// Rotary Encoder to decrease volume



/*
 * IRremote: IRrecvDemo - demonstrates receiving IR codes with IRrecv
 * An IR detector/demodulator must be connected to the input RECV_PIN.
 * Version 0.1 July, 2009
 * Copyright 2009 Ken Shirriff
 * http://arcfn.com
 */

#include <IRremote.h>

int RECV_PIN = 3;
IRrecv irrecv(RECV_PIN);

decode_results results;
unsigned int IRdata = 0;       		// variable for received character
unsigned int IRkey; 				// The unique code of the remote key


/********************************************************************************************************/


#define B 0xFF             			// The character for a completely filled box
#define A 0x20             			// The character for blank

// The routine to create the custom characters in the LCD

  // A 1 in the binary representation of the character means it is filled in
  // characters are 5 pixels wide by 8 pixels tall

  // We need 7 custom characters for the OPUS DAC display.
  // (Custom character 0 doesn't work in the Web4Robot LCD)

  // Define Custom Characters

byte cc0[8] = {     // Custom Character 0
  B00000,
  B00111,
  B01111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

byte cc1[8] = {     // Custom Character 1
  B11100,
  B11110,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

byte cc2[8] = {    // Custom Character 2
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
  B00000,
  B00000
};

byte cc3[8] = {    // Custom Character 3
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

byte cc4[8] = {   // Custom Character 4
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B01111,
  B00111
};

byte cc5[8] = {    // Custom Character 5
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
  B00000
};

byte cc6[8] = {    // Custom Character 6
  B00000,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
  B00000
};

byte cc7[8] = {     // Custom Character 7
  B00000,
  B11100,
  B11110,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

// Function to send custom characters to the display's RAM
void DefineCustomChar(){
  lcd.createChar(0,cc0);  			// cc0 becomes character 0
  lcd.createChar(1,cc1);  			// cc1 becomes character 1
  lcd.createChar(2,cc2);  			// cc2 becomes character 2
  lcd.createChar(3,cc3);  			// cc3 becomes character 3
  lcd.createChar(4,cc4);  			// cc4 becomes character 4
  lcd.createChar(5,cc5);  			// cc5 becomes character 5
  lcd.createChar(6,cc6);  			// cc6 becomes character 6
  lcd.createChar(7,cc7);  			// cc7 becomes character 7
};

// Array index into parts of big numbers. Each number consists of 9 custom characters
// in a 3x3 matrix. To print a number, you use the array index corresponding to the number
// times 3. For example to print the number 5, you will print bn1[15], bn1[16] and bn1[17]
// for the first row of the large number, and then bn2[15], bn2[16] and bn2[17] and so on.
//
//            0      1      2      3      4      5      6      7      8      9
char bn1[]={B,2,1, 2,1,A, 2,2,1, 2,2,1, 0,A,B, B,2,2, B,2,2, 2,2,B, B,2,1, B,2,1};
char bn2[]={B,A,B, A,B,A ,0,6,5, A,2,1, 5,6,B, 2,2,1, B,6,7, A,0,5, B,6,B, 5,6,B};
char bn3[]={4,3,B, 3,B,3, B,3,3, 4,3,B, A,A,B, 4,3,B, 4,3,B, A,B,A, 4,3,B, A,A,B};

void printOneNumber(uint8_t digit)
{
  // Print position is hardcoded

  // Line 1 of the one digit number
  lcd.setCursor(0,1);
  delay(5);             				// add delay after each position change for slow LCD
  lcd.write(bn1[digit*3]);
  lcd.write(bn1[digit*3+1]);
  lcd.write(bn1[digit*3+2]);

  // Line 2 of the one-digit number
  lcd.setCursor(-4,2);
  delay(5);             				// add delay after each position change for slow LCD
  lcd.write(bn2[digit*3]);
  lcd.write(bn2[digit*3+1]);
  lcd.write(bn2[digit*3+2]);

  // Line 3 of the one-digit number
  lcd.setCursor(-4,3);
  delay(5);             				// add delay after each position change for slow LCD
  lcd.write(bn3[digit*3]);
  lcd.write(bn3[digit*3+1]);
  lcd.write(bn3[digit*3+2]);
}

// Functions for printing two large digits. Value is the column number
// and number is the number to print. Works from 00-99

void printTwoNumber(byte value, byte number){

  lcd.setCursor(value-4,1);  			// Printing line 1 of the two-digit number
  delay(50);             				// add delay after each position change for slow LCD
  lcd.write(bn1[(number/10)*3]);
  lcd.write(bn1[(number/10)*3+1]);
  lcd.write(bn1[(number/10)*3+2]);
  lcd.write(A); // Blank
  lcd.write(bn1[(number%10)*3]);
  lcd.write(bn1[(number%10)*3+1]);
  lcd.write(bn1[(number%10)*3+2]);

  lcd.setCursor(value-8,2);  			// Printing line 2 of the two-digit number
  delay(50);             				// add delay after each position change for slow LCD
  lcd.write(bn2[(number/10)*3]);
  lcd.write(bn2[(number/10)*3+1]);
  lcd.write(bn2[(number/10)*3+2]);
  lcd.write(A); // Blank
  lcd.write(bn2[(number%10)*3]);
  lcd.write(bn2[(number%10)*3+1]);
  lcd.write(bn2[(number%10)*3+2]);

  lcd.setCursor(value-8,3);  			// Printing line 3 of the two-digit number
  delay(50);             				// add delay after each position change for slow LCD
  lcd.write(bn3[(number/10)*3]);
  lcd.write(bn3[(number/10)*3+1]);
  lcd.write(bn3[(number/10)*3+2]);
  lcd.write(A); // Blank
  lcd.write(bn3[(number%10)*3]);
  lcd.write(bn3[(number%10)*3+1]);
  lcd.write(bn3[(number%10)*3+2]);

};
/**************************************************************************/
// The write to WM8741 DAC routine
/**************************************************************************/
void opuswritereg(uint8_t regaddr,uint8_t regval)
{
  Wire.beginTransmission(0x1B);
  Wire.write(regaddr);
  Wire.write(regval);
  Wire.endTransmission();
}
/**************************************************************************/
// Interrupt service routine for rotary encoder. Determines direction.
/**************************************************************************/

volatile byte volUp=0;  				// flags for the interrupt routine
volatile byte volDown=0;

void decoder()
{
  if (digitalRead(2) == digitalRead(4))
  {
    volUp = 1;  						//	if on interrupt the encoder channels are the same,
										// direction is clockwise
  }
  else
  {
    volDown = 1;  						//	if they are not the same, direction is ccw
  }
}
/**************************************************************************/
// interrupt service routine for remote
/**************************************************************************/

volatile byte remoteOn = 0;  			// 1 means remote has been pressed

void remoting()
{
  remoteOn=1;
}

// Declaring some more variables

int currVol=DEFAULTVOL; 				// this needs to be at least 10 bits
byte filter=0;          				// variable for current filter
byte reg0val=0;         				// variable for volume lower bits
byte reg1val=0;         				// variable for volume upper bits
byte volMuteState=0;     				// State for volume Mute Control. =1 is Muteded
byte BC=0;              				// State for selecting contrast or brightness
                        				// BC=0, no adjustment, BC=1" Brighness
                        				// BC=2: contrast
int brightness=0;
int contrast=0;



/**************************************************************************/
/*  Clear the display and run setup to begin                              */
/**************************************************************************/

void setup() {
 // lcd.init();     					// Initializing the LCD, clears the display
  lcd.begin(16, 4);    					// Set up the LCD's number of columns and rows:

 // lcd.setDelay(0,0); 					// Change de delay in the LCD library
  DefineCustomChar();  					// Define large characters

  Serial.begin(9600);
  irrecv.enableIRIn(); 					// Start the receiver

  Wire.begin();   						// Joining the I2C bus as master

  pinMode(FILTERPIN, INPUT);      		// Button to select Input fiter
  digitalWrite(FILTERPIN, HIGH);  		// Enable pull-up resistor

  pinMode(VOLUPPIN, INPUT);       		// Button or Encoder pin for volume up
  digitalWrite(VOLUPPIN, HIGH);   		// Enable pull-up resistor

  pinMode(VOLDOWNPIN, INPUT);     		// Button or Encoder pin for volume down
  digitalWrite(VOLDOWNPIN, HIGH); 		// Enable pull-down resistor

  pinMode(VOLMutePIN, INPUT);      		// Button or switch for volume Mute Control
  digitalWrite(VOLMutePIN, HIGH);  		// Enable pull-up resistor

  // pinMode(REMOTEPIN, INPUT);     	// Pin for IR Receiver
  // digitalWrite(REMOTEPIN, HIGH); 	// Enable high as specified by datasheet

  // Print the welcome message and other labels to the LCD

  lcd.clear();
  lcd.setCursor(0,0);
  delay(50);             				// add delay after each position change for slow LCD
  lcd.print("HIFIDUINO");
  lcd.setCursor(0,1);
  delay(50);             				// add delay after each position change for slow LCD
  lcd.print("W8741 STEREO DAC");
  lcd.setCursor(-4,2);
  delay(50);             				// add delay after each position change for slow LCD
  lcd.print("Ver B0.01.06");
  lcd.setCursor(-4,3);
  delay(50);             				// add delay after each position change for slow LCD
  lcd.print("April 26, 2012");

/**************************************************************************/
/*  signon complete, delay 2 sec for credits, then go to operate screen    */
/**************************************************************************/
  delay(2000);

  lcd.clear();

  opuswritereg(REG9,RESET);   			// Resetting the DAC just in case
  opuswritereg(REG4,MUTE);    			// MUTE (and vol setup)

  // The following 4 lines is to set up the default volume
  // To set up the volume you need to write to at least 2 registers

  reg0val=(DEFAULTVOL & 0x1F);     		// The lower 5 bits goes to reg 0
  opuswritereg(REG0,reg0val);      		// Writing to reg 0
  reg1val=((DEFAULTVOL>>5)| 0x20); 		// The upper 5 bits goes to reg 1
  opuswritereg(REG1,reg1val);      		// Writing to reg 1

  opuswritereg(REG6,FILTER1);  			// I'm setting the default filter at power on
  opuswritereg(REG7,PCMHIGH);  			// Setting up for PCM and high sampling rate because I have
                               			// a reclocker and the output is 192K to the DAC
  opuswritereg(REG4,VOLCNTR);  			// Setting up volume control. Unmute

  //attachInterrupt(0, decoder, CHANGE); // ISR for rotary encoder
  attachInterrupt(0, decoder, RISING);  // ISR for rotary encoder
  attachInterrupt(1, remoting, RISING); // ISR for remote IR sensor

  lcd.setCursor(0,0);
  delay(5);             				// add delay after each position change for slow LCD
  // print header for digital filter and current filter
  lcd.print("LnS");
  //printTwoNumber(13,1);
  printOneNumber(1);

  // print header for volume and volume
  lcd.setCursor(8,0);
  delay(5);             				// add delay after each position change for slow LCD
  lcd.print("-dB  VOL");
  printTwoNumber(13, currVol/8);

  // Print the rest of the display
  lcd.setCursor(-1,2);
  delay(5);             				// add delay after each position change for slow LCD
  lcd.print("IN");
  lcd.write(0xA5);
  lcd.print("PCM");
  lcd.setCursor(3,1);
  delay(5);             				// add delay after each position change for slow LCD
  lcd.print("RT");
  lcd.write(0xA5);
  lcd.print("192");

}

/******************************************************************************/
/* -----------------------------Main Loop ------------------------------------*/
/******************************************************************************/

void loop()
{
/*   this is the decoder                                                */

  if (irrecv.decode(&results)) {
    irrecv.resume();                	// Receive the next value
    IRdata=results.value ;

/* set up and print the masked byte of the received long word          */
/* takes the place of the old serial print command for testing         */
/* to find the codes for each key needed                               */
//  Serial.println(IRdata, HEX);
//
//    		lcd.setCursor(0,0);
//    		delay(5);
//    		lcd.clear();
//    		delay(5);
//    		lcd.print(IRDATA, HEX);
//    		delay(500);

	IRkey = 0;					   		 // clear the previous code to prevent false repetition

//   shift as necessary and Mask off IRdata to leave the lower byte (256 codes)
//   as IRkey. The codes will be converted a standard sequence for the case
//   handeling.


	if (RC6)
		{
	   IRkey = (IRdata & 0x0FF);
			}
			else if (SONY)
			{
	   		IRkey = ((IRdata >> 4)& 0x0FF);
			}
			else if (APPLE)
			{
	   		IRkey = ((IRdata >> 8)& 0x0FF);
			}

/*    add more else if statements here for additional IR Remote Controls   */

			else {
		}									// code for key now masked off so it is an 8bit code.

/*   Dey decoding done here   */
/* comment out the decoder here while testing new code sets to prevent screen refresh*/


    	if (IRkey == CODE1)
		{
		IRkey = 1;
		}

		else if (IRkey == CODE2)
		{
		IRkey = 2;
		}

		else if (IRkey == CODE3)
		{
		IRkey = 3;
		}
		else if (IRkey == CODE4)
		{
		IRkey = 4;
		}

		else if (IRkey == CODE5)
		{
		IRkey = 5;
		}

		else if (IRkey == CODEDN)
		{
		IRkey = 6;
		}

		else if (IRkey == CODEUP)
		{
		IRkey = 7;
		}

		else if (IRkey == CODEMU)
		{
		IRkey = 8;
		}
 	   delay(100);
	}


/******************************************************************************/
/*  Check for filter button pushed                                            */
/******************************************************************************/
// The following code will select the next filter if the button is pushed
// and it is repeated for the remote above because it is easier than
//	modifying this code

  if (digitalRead(FILTERPIN)==0)  			// When the filter button is pushed
  {
    filter++;
    filter=filter%5;
        switch(filter){

      case 0:
      lcd.setCursor(0,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print("LnS");
      opuswritereg(REG6,FILTER1);
      printOneNumber(1);
      break;
      case 1:
      lcd.setCursor(0,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print("MnS");
      opuswritereg(REG6,FILTER2);
      printOneNumber(2);
      break;
      case 2:
      lcd.setCursor(0,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print("BrK");
      opuswritereg(REG6,FILTER3);
      printOneNumber(3);
      break;
      case 3:
      lcd.setCursor(0,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print("MnA");
      opuswritereg(REG6,FILTER4);
      printOneNumber(4);
      break;
      case 4:
      lcd.setCursor(0,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print("LnA");
      opuswritereg(REG6,FILTER5);
      printOneNumber(5);
      break;
    }
    delay(300);  							// Used to avoid reading the button more than one during the click
  }


/*  Check for Rotary Encoder operation                                        */
/******************************************************************************/
/*   The following is to adjust the volume down (larger number)               */
/******************************************************************************/

  while(volDown==1)  						// While there is CCW motion in the rotary encoder
                     						// This means decrease volume -increase attenuation
  {
    delay(5);   							// Debouncing delay - maybe in the wrong place...
    volDown=0;  							// Reset the flag
    if (currVol<MINVOL) 					// If current volume is not at MIN (attn < maxAttn)
                        					// then decrease volume (increase attenuation)
    {
      currVol=currVol+8;                    // Increase attenuation 1 dB
      reg0val=(((byte)currVol) & 0x1F);     // Calculate value for reg0
      opuswritereg(REG0,reg0val);           // Write to reg0
      reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
      opuswritereg(REG1,reg1val);           // Write to reg1

      printTwoNumber(13, currVol/8);

      if (volMuteState==1)                	// If in Mute state, reset state value
      {
        volMuteState=0;
        lcd.setCursor(12,0);
  delay(5);             					// add delay after each position change for slow LCD
        lcd.print(" VOL");
      }
    }
  }

/**********************************************************************************/
/*   The following is to adjust the volume up via Rotary Encoder (smaller numbers)*/
/**********************************************************************************/

  while(volUp==1)  							// While there is cw motion in rotary encoder
  {
    delay(5); 								// Debouncing delay - maybe in the wrong place...
    volUp=0;  								// clear the flag
    if (currVol>MAXVOL)   					// If current volume not at MAX (attn>minAttn)
                          					// increase volume (reduce attennuation)
    {
      currVol=currVol-8;                	// Decrease attenuation 1 dB
      reg0val=(((byte)currVol) & 0x1F); 	// Calculate value for reg0
      opuswritereg(REG0,reg0val);       	// Write to reg0
      reg1val=((currVol>>5)| 0x20);     	// Calculate value for reg1
      opuswritereg(REG1,reg1val);       	// Write to reg1

      printTwoNumber(13, currVol/8);

      if (volMuteState==1)               	// If in Mute state, reset state value
      {
        volMuteState=0;
        lcd.setCursor(12,0);
  delay(5);             					// add delay after each position change for slow LCD
        lcd.print(" VOL");
      }
    }
  }

/*************************************************************************************/
/*The following is for the volume Muting feature via the button on the Rotary Encoder*/
/*************************************************************************************/

  if (digitalRead(VOLMutePIN)==0) 			// When pressing the volume Mute button
  {
    if (volMuteState==0)  					//if not Muted, then Mute the volume to min vol
    {
      reg0val=(((byte)(currVol+MuteVOL)) & 0x1F);
      opuswritereg(REG0,reg0val);
      reg1val=(((byte)((currVol+MuteVOL)>>5))| 0x20);
      opuswritereg(REG1,reg1val);
      printTwoNumber(13, (currVol+MuteVOL)/8);
      volMuteState=1;     					// set state to Muted
      lcd.setCursor(12,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print("Mute");
    }
    else                 					// if Muted, then return to current volume
    {
      reg0val=(((byte)currVol) & 0x1F);
      opuswritereg(REG0,reg0val);
      reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
      opuswritereg(REG1,reg1val);
      printTwoNumber(13, currVol/8);
      volMuteState=0;    					// set state to unMuted.
      lcd.setCursor(12,0);
  delay(5);             					// add delay after each position change for slow LCD
      lcd.print(" VOL");
    }
    delay(300); 							// Used to avoid reading the button more than one during the click
  }

/********************************************************************************/
/* IR Remote Switch statement for Vol up, Vol Dn, Mute//Un-Mute                 */
/********************************************************************************/
//
/*****************************************************************************/
/* IR Remote code decoded, so use it to change configuration                 */
/******************************************************************************/
//
//	1. Filter selection 	1-5
//	2. Volume up and down 	6,7
//	3. Mute/Un-Mute 		8
//
//
// The following code will select the next filter if the the remote is used
// and it is coppied from the button code
//
//  if (digitalRead(FILTERPIN)==0)  			// When the filter button is pushed
//  {
//    filter++;
//    filter=filter%5;
        switch(IRkey){

      case 1:
      lcd.setCursor(0,0);
  delay(5);             						// add delay after each position change for slow LCD
      lcd.print("LnS");
      opuswritereg(REG6,FILTER1);
      printOneNumber(1);
      break;
      case 2:
      lcd.setCursor(0,0);
  delay(5);             						// add delay after each position change for slow LCD
      lcd.print("MnS");
      opuswritereg(REG6,FILTER2);
      printOneNumber(2);
      break;
      case 3:
      lcd.setCursor(0,0);
  delay(5);             						// add delay after each position change for slow LCD
      lcd.print("BrK");
      opuswritereg(REG6,FILTER3);
      printOneNumber(3);
      break;
      case 4:
      lcd.setCursor(0,0);
  delay(5);             						// add delay after each position change for slow LCD
      lcd.print("MnA");
      opuswritereg(REG6,FILTER4);
      printOneNumber(4);
      break;
      case 5:
      lcd.setCursor(0,0);
  delay(5);             						// add delay after each position change for slow LCD
      lcd.print("LnA");
      opuswritereg(REG6,FILTER5);
      printOneNumber(5);
      break;

      // The following for volume
      //case KEYVOLUP:
      case 6:       							// the up key, we will use for volume up
      if (currVol>MAXVOL) 						// If volume not at max vol, then increse vol
                          						// volume is increased by decreasing the attenuation value
        {
        currVol=currVol-8;                		// Decrease attenuation 1 dB
        reg0val=(((byte)currVol) & 0x1F); 		// Calculate value for reg0
        opuswritereg(REG0,reg0val);       		// Write to reg0
        reg1val=((currVol>>5)| 0x20);     		// Calculate value for reg1
        opuswritereg(REG1,reg1val);       		// Write to reg1

        printTwoNumber(13, currVol/8);

        if (volMuteState==1)               		// If in Mute state, reset state value
        {
          volMuteState=0;
          lcd.setCursor(12,0);
  delay(5);             						// add delay after each position change for slow LCD
          lcd.print(" VOL");

        }
      }
      break;

      //case KEYVOLDOWN:
      case 7:             						// 1the down key, we will use for volume down
      if (currVol<MINVOL)  						// If not at min volume, then can decrease volume
        {                  						// volume is decreased by increasing the attenuation value
        currVol=currVol+8;                    	// Increase attenuation 1 dB
        reg0val=(((byte)currVol) & 0x1F);     	// Calculate value for reg0
        opuswritereg(REG0,reg0val);           	// Write to reg0
        reg1val=(((byte)(currVol>>5))| 0x20); 	// Calculate value for reg1
        opuswritereg(REG1,reg1val);           	// Write to reg1

        printTwoNumber(13, currVol/8);

        if (volMuteState==1)                	// If in Mute state, reset state value
        {
          volMuteState=0;
          lcd.setCursor(12,0);
  delay(5);             						// add delay after each position change for slow LCD

        }
      }
      break;

	  //case MUTE:
      case 8:
        {
      if (volMuteState==0)  //if not Muted, then Mute the volume to min vol
          {
          reg0val=(((byte)(currVol+MuteVOL)) & 0x1F);
          opuswritereg(REG0,reg0val);
          reg1val=(((byte)((MuteVOL+currVol)>>5))| 0x20);
          opuswritereg(REG1,reg1val);
          printTwoNumber(13, (MuteVOL+currVol)/8);
          volMuteState=1;     					// set state to Muted
          lcd.setCursor(12,0);
  delay(5);             						// add delay after each position change for slow LCD
          lcd.print("Mute");
          }
        else                 // if Muted, then return to current volume
          {
          reg0val=(((byte)currVol) & 0x1F);
          opuswritereg(REG0,reg0val);
          reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
          opuswritereg(REG1,reg1val);
          printTwoNumber(13, currVol/8);
          volMuteState=0;    					// set state to unMuted.
          lcd.setCursor(12,0);
  delay(5);             						// add delay after each position change for slow LCD
          lcd.print(" VOL");
          }
        }
	  }
      delay(500); 								// Used to avoid reading the button more than one during the click
	IRkey = 0;
 }
// }
